searchData={"items":[{"type":"module","title":"LiveViewEvents","doc":"Add `use LiveViewEvents` to the module you want to use any\nof the features of `LiveViewEvents` in.\n\nFor more info about sending and receiving events, see `LiveViewEvents.Notify`.","ref":"LiveViewEvents.html"},{"type":"module","title":"LiveViewEvents.Notify","doc":"Functions to send messages in the server including to live components\nand handle them.","ref":"LiveViewEvents.Notify.html"},{"type":"module","title":"Example - LiveViewEvents.Notify","doc":"In our view, we are going to have two components. The first one is a `Sender`\nthat can send an event to whatever is being passed. Let's dig into the\nimplementation:\n\n    defmodule MyAppWeb.Components.Sender do\n      use MyAppWeb, :live_component\n      use LiveViewEvents\n\n      def mount(socket) do\n        socket = assign(socket, :notify_to, :self)\n\n        {:ok, socket}\n      end\n\n      def render(assigns) do\n        ~H[ Send event ]\n      end\n\n      def handle_event(\"clicked\", _params, socket) do\n        notify_to(socket.assigns.notify_to, :sender_event, :rand.uniform(100))\n        {:noreply, socket}\n      end\n    end\n\nThis component will send a `:sender_event` message to whatever we pass to the `notify_to`\nattribute. It sends a random number between 0 and up to 100 as parameter.\n\nLet's dig into the receiver:\n\n    defmodule MyAppWeb.Components.Receiver do\n      use MyAppWeb, :live_component\n      use LiveViewEvents\n\n      def mount(socket) do\n        socket = assign(socket, :messages, [])\n\n        {:ok, socket}\n      end\n\n      def update(assigns, socket) do\n        socket = handle_info_or_assign(socket, assigns)\n\n        {:ok, socket}\n      end\n\n      def render(assigns), do: ~H[  <%= m %>  ]\n\n      def handle_info({:sender_event, num}, socket) do\n        {:noreply, update(socket, :messages, &[num | &1])}\n      end\n    end\n\nThis component will receive messages and handle them in `handle_info/2` because\nit is using `handle_info_or_assign/2` in their [`LiveComponent.update/2`](`c:Phoenix.LiveComponent.update/2`).\nIt will add the received messages to `socket.assigns.messages` and display\nthem. As the reader can see, it is pattern matching against `:sender_event` messages.\nWhen `notify_to/3` is used, the message sent is a tuple containing the event name\nas first element, and the params as second the second element.\n\nFinally, let's take a look at what the live view template would need to look like\nfor this to work:\n\n     \n      <.live_component\n        module={MyAppWeb.Components.Sender}\n        id=\"sender\"\n        notify_to={{MyAppWeb.Components.Receiver, \"receiver\"}}\n        />\n      <.live_component module={MyAppWeb.Components.Receiver} id=\"receiver\" />\n     \n\nIn this template, we set `notify_to` to the tuple `{MyAppWeb.Components.Receiver, \"receiver\"}`.\nThe first element of the tuple is the live component module and the second is the id.\nOptionally, the tuple can contain an extra first element that needs to be a PID. Though\nthis might not be useful in the application code (there are way better ways to send events\nbetween processes), it is quite useful when testing. When testing a [`LiveView`](`Phoenix.LiveView`),\nit creates a new process for it. Its PID can be accessed through `view.pid`.","ref":"LiveViewEvents.Notify.html#module-example"},{"type":"macro","title":"LiveViewEvents.Notify.handle_info_or_assign/2","doc":"Use this macro instead of the default `assigns(socket, assign)` in\n`c:Phoenix.LiveComponent.update/2`.\n\nThis will detect if `c:Phoenix.LiveComponent.update/2` is being called\nbecause of an event send with either `notify_to/2` or `notify_to/3`\nand handle it with `c:Phoenix.LiveView.handle_info/2`. Otherwise,\nit will assign the given `assigns` to `socket`.\n\nIf there is no `c:Phoenix.LiveView.handle_info/2` defined in the\ncomponent, sending an event won't raise an error but if there is one\ndefined and it cannot handle the received message, it will.\n\nFurthermore, if the handler returns anything that is not a tuple\n`{:noreply, socket}`, it'll raise an exception too.","ref":"LiveViewEvents.Notify.html#handle_info_or_assign/2"},{"type":"macro","title":"Why using `c:Phoenix.LiveView.handle_info/2` in components? - LiveViewEvents.Notify.handle_info_or_assign/2","doc":"In one word: consistency. Messages coming from the client are\nhandled by [LiveView.handle_event/3](`c:Phoenix.LiveView.handle_event/3`)\nin live views or by [`LiveComponent.handle_event/3`](`c:Phoenix.LiveComponent.handle_event/3`)\nin live components.\nMessages sent from the server are currently being handled by\n[`LiveView.handle_info/2`](`c:Phoenix.LiveView.handle_info/2`) in live views,\nwith no official way to do this but the __hack__ this library is based on.\n\nThe hack is basically send an update with [`LiveView.send_update/3`](`Phoenix.LiveView.send_update/3`)\nand handle it in [`LiveComponent.update/2`](`c:Phoenix.LiveComponent.update/2`).","ref":"LiveViewEvents.Notify.html#handle_info_or_assign/2-why-using-c-phoenix-liveview-handle_info-2-in-components"},{"type":"function","title":"LiveViewEvents.Notify.notify_to/2","doc":"`notify_to/2` accepts a target and a message name. The target can be any of:\n\n- `nil` would make it be a noop.\n- `:self` to send to `self()`.\n- A PID.\n- A tuple of the form `{Module, \"id\"}` to send a message to a [`LiveView.Component`](`Phoenix.LiveView.Component`) in the same process.\n- A tuple of the form `{pid, Module, \"id\"}` to send a message to a [`LiveView.Component`](`Phoenix.LiveView.Component`) in a different process.\n\nThe event send will take the form of `{message, %{}}`.","ref":"LiveViewEvents.Notify.html#notify_to/2"},{"type":"function","title":"LiveViewEvents.Notify.notify_to/3","doc":"`notify_to/3` behaves like `notify_to/2` but accepting some extra parameters as the third arguments.\nIn this case, the message sent would be a tuple with the `message` as first element and `params` as the\nsecond one.","ref":"LiveViewEvents.Notify.html#notify_to/3"},{"type":"extras","title":"Overview","doc":"# LiveViewEvents\n\n![Elixir CI](https://github.com/DockYard/live_view_events/actions/workflows/elixir-ci.yml/badge.svg)\n\nLiveViewEvents provides a set of tools to send events between components.","ref":"readme.html"},{"type":"extras","title":"Installation - Overview","doc":"Add `live_view_events` to your list of dependencies in `mix.exs` like:\n\n```elixir\ndef deps do\n  [\n    {:live_view_events, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Overview","doc":"Add `use LiveViewEvents` whenever you want to use any of the features of the libraries.\n\n#","ref":"readme.html#usage"},{"type":"extras","title":"Sending events to LiveView components - Overview","doc":"You can send events to LiveView components by using `notify_to/2` or `notify_to/3` (the only\ndifference being that the latter sends some extra params). These functions accept a target as\nfirst argument and a message name as second. Targets can be any of:\n\n  - `nil` makes the function call be a noop. Useful if you want to only send a message if an explicit target has been passed in.\n  - `:self` to send to `self()`.\n  - A PID.\n  - A tuple of the form `{Module, \"id\"}` to send a message to a `LiveView.Component` in the same process.\n  - A tuple of the form `{pid, Module, \"id\"}` to send a message to a `LiveView.Component` in a different process.\n\nYou an handle these messages on the `handle_info/2` callback on\nyour LiveView components. For being able to do so, you need to\nuse the `handle_info_or_assign/2` macro on the `update/3` callback of your component.","ref":"readme.html#sending-events-to-liveview-components"},{"type":"extras","title":"Example - Overview","doc":"In our view, we are going to have two components. The first one is a `Sender`\nthat can send an event to whatever is being passed. Let's dig into the\nimplementation:\n\n```elixir\ndefmodule MyAppWeb.Components.Sender do\n  use MyAppWeb, :live_component\n  use LiveViewEvents\n\n  def mount(socket) do\n    socket = assign(socket, :notify_to, :self)\n\n    {:ok, socket}\n  end\n\n  def render(assigns) do\n    ~H[ Send event ]\n  end\n\n  def handle_event(\"clicked\", _params, socket) do\n    notify_to(socket.assigns.notify_to, :sender_event, :rand.uniform(100))\n    {:noreply, socket}\n  end\nend\n```\n\nThis component will send a `:sender_event` message to whatever we pass to the `notify_to`\nattribute. It sends a random number between 0 and up to 100 as parameter.\n\nLet's dig into the receiver:\n\n```elixir\ndefmodule MyAppWeb.Components.Receiver do\n  use MyAppWeb, :live_component\n  use LiveViewEvents\n\n  def mount(socket) do\n    socket = assign(socket, :messages, [])\n\n    {:ok, socket}\n  end\n\n  def update(assigns, socket) do\n    socket = handle_info_or_assign(socket, assigns)\n\n    {:ok, socket}\n  end\n\n  def render(assigns),\n    do: ~H[  <%= m %>  ]\n\n  def handle_info({:sender_event, num}, socket) do\n    {:noreply, update(socket, :messages, &[num | &1])}\n  end\nend\n```\n\nThis component will receive messages and handle them in `handle_info/2` because\nit is using `handle_info_or_assign/2` in their `LiveComponent.update/2`.\nIt will add the received messages to `socket.assigns.messages` and display\nthem. As the reader can see, it is pattern matching against `:sender_event` messages.\nWhen `notify_to/3` is used, the message sent is a tuple containing the event name\nas first element, and the params as second the second element.\n\nFinally, let's take a look at what the live view template would need to look like\nfor this to work:\n\n```heex\n \n  <.live_component\n    module={MyAppWeb.Components.Sender}\n    id=\"sender\"\n    notify_to={{MyAppWeb.Components.Receiver, \"receiver\"}}\n  />\n  <.live_component module={MyAppWeb.Components.Receiver} id=\"receiver\" />\n \n```\n\nIn this template, we set `notify_to` to the tuple `{MyAppWeb.Components.Receiver, \"receiver\"}`.\nThe first element of the tuple is the live component module and the second is the id.\nOptionally, the tuple can contain an extra first element that needs to be a PID. Though\nthis might not be useful in the application code (there are way better ways to send events\nbetween processes), it is quite useful when testing. When testing a `LiveView`\nit creates a new process for it. Its PID can be accessed through `view.pid`.","ref":"readme.html#example"},{"type":"extras","title":"Why are messages without params still receiving empty params? - Overview","doc":"To normalize how to handle messages. If we could have `handle_info(\"message_name\", _socket)`\nor `handle_info({\"message_name\", _params}, socket)`, you would need to think about whether\nthe message contains params or not and then use a tuple. With this normalization, you only\ncare about the message name and whether you need the params or not. Also, after using similar\napproaches in the past, sometimes you want to send params and sometimes you don't need to.\nIn cases like these, it is helpful not having a different behaviour notifying an event without\nparams and sending an event with _empty_ params.","ref":"readme.html#why-are-messages-without-params-still-receiving-empty-params"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}